@@ -23,13 +23,13 @@
 
 #include "actions.h"
 #include "codeine.h"
-#include "debug.h"
 #include "partToolBar.h"
 #include "videoWindow.h"
 
 #include <KAction>
 #include <KAboutData>
 #include <KActionCollection>
+#include <KDebug>
 #include <KMenu>
 #include <KParts/GenericFactory>
 #include <KToggleAction>
@@ -69,12 +69,9 @@ namespace Dragon
             ac->addAction( sliderAction->objectName(), sliderAction );
             toolBar->addAction( sliderAction );
         }
-        connect( engine(), SIGNAL( stateUpdated( Phonon::State, Phonon::State ) ),
-                 this, SLOT( engineStateChanged( Phonon::State ) ) );
-
+        connect( engine(), SIGNAL( stateChanged( Phonon::State ) ), this, SLOT( engineStateChanged( Phonon::State ) ) );
         videoWindow()->setContextMenuPolicy( Qt::CustomContextMenu );
-        connect( videoWindow(), SIGNAL( customContextMenuRequested(const QPoint &) ),
-                 this, SLOT( videoContextMenu(const QPoint &) ) );
+        connect( videoWindow(), SIGNAL( customContextMenuRequested() ), this, SLOT( videoContextMenu() ) );
 
         widget()->setLayout( layout );
     }
@@ -88,8 +85,7 @@ namespace Dragon
     bool
     Part::openUrl( const KUrl &url )
     {
-        DEBUG_BLOCK
-        debug() << "playing " << url;
+        kDebug() << "playing " << url;
         bool ret = videoWindow()->load( m_url = url );
         videoWindow()->play();
         return ret;
@@ -98,7 +94,6 @@ namespace Dragon
     bool
     Part::closeUrl()
     {
-        DEBUG_BLOCK
         m_url = KUrl();
         videoWindow()->stop();
         return true;
@@ -120,7 +115,6 @@ namespace Dragon
     bool 
     Part::openFile() //pure virtual in base class
     {
-        DEBUG_BLOCK
         return false; 
     }
     
